import cv2
import numpy as np
import glob
from scipy.io import savemat
import matplotlib.pyplot as plt
import os
import ctypes
import pupil_apriltags as apriltag  # 确保使用正确的库
from aprilgrid import Detector
from .calibrate import generate_3d_points, show_detect_result_new, show_detect_result, visualize_3d_points

# 读取文件夹中的所有棋盘格图像
# images = glob.glob('calibration/screen_calibration/*.bmp')


def calibrate_screen_chessboard(screen_img_path, cam_mtx, cam_dist, chessboard_size, square_size, debug=False):
    # Prepare object points for chessboard corners in world coordinates
    objp = np.zeros((chessboard_size[0] * chessboard_size[1], 3), np.float32)
    objp[:, :2] = np.mgrid[0:chessboard_size[0], 0:chessboard_size[1]].T.reshape(-1, 2)
    objp *= square_size
    # import pdb; pdb.set_trace()

    objpoints = []  # 三维点
    imgpoints = []  # 二维点

    #print('cam_mtx = ', cam_mtx)

    for img_path in screen_img_path:
        
        image_data = np.fromfile(img_path, dtype=np.uint8)
        img = cv2.imdecode(image_data, cv2.IMREAD_COLOR)
       
        # import pdb; pdb.set_trace()
        dst = cv2.undistort(img, cam_mtx, cam_dist, None, None)
        gray = cv2.cvtColor(dst, cv2.COLOR_BGR2GRAY)
        # alpha = 3  # 对比度保持不变
        # beta = 0    # 增加亮度
        # bright_image = cv2.convertScaleAbs(gray, alpha=alpha, beta=beta)
        # cv2.namedWindow('bright_image', 0)
        # cv2.imshow('bright_image', bright_image)
        # cv2.namedWindow('gray', 0)
        # cv2.imshow('gray', gray)
        # cv2.waitKey(0)
        # cv2.destroyAllWindows()
        flag = cv2.CALIB_CB_EXHAUSTIVE | cv2.CALIB_CB_ACCURACY
        ret, corners = cv2.findChessboardCornersSB(gray, chessboard_size, flag)
        #ret, corners = cv2.findChessboardCorners(gray, chessboard_size, flags=cv2.CALIB_CB_FAST_CHECK)
        if ret:
           # print('screen img_path = ',img_path)
            objpoints.append(objp)
            criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 30, 0.001)
            corners_refined = cv2.cornerSubPix(gray, corners, (11, 11), (-1, -1), criteria)
            imgpoints.append(corners_refined)

            if debug:
                img = cv2.drawChessboardCorners(img, chessboard_size, corners, ret)
                # 在角点图像上标记坐标原点（第一个角点）
                origin = tuple(corners[0].ravel().astype(int))
                cv2.putText(img, '(0,0)', origin, cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)

                # 添加 x 轴箭头 (右方向)
                end_x = tuple(corners[1].ravel().astype(int))  # 选择横向的下一个角点
                cv2.arrowedLine(img, origin, end_x, (255, 255, 255), 2, tipLength=0.2)
                cv2.putText(img, 'X', (end_x[0] + 10, end_x[1]), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)

                # 添加 y 轴箭头 (下方向)
                end_y = tuple(corners[chessboard_size[0]].ravel().astype(int))  # 选择纵向的下一个角点
                cv2.arrowedLine(img, origin, end_y, (255, 255, 255), 2, tipLength=0.2)
                cv2.putText(img, 'Y', (end_y[0], end_y[1] + 30), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)
                cv2.namedWindow('screen', 0)
                cv2.imshow('screen', img)
                cv2.waitKey(100)
                
    # import pdb; pdb.set_trace()
    cv2.destroyAllWindows()

    objpoints = np.array(objpoints)
    imgpoints = np.array(imgpoints)
    # import pdb; pdb.set_trace()
    # if debug:
    #     # 使用两幅子图绘制 objp 和 corners 的点
    #     fig, ax = plt.subplots(1, 2, figsize=(12, 6))

    #     # 第一个子图：objp 点
    #     ax[0].scatter(objpoints[-1, :, 0], objpoints[-1, :, 1], color='blue', label='objp (Object Points)')
    #     for i in range(len(objpoints[0])):
    #         ax[0].text(objpoints[-1, i, 0], objpoints[-1, i, 1], f'{i}', color='blue', fontsize=12)
    #     ax[0].set_title('Object Points (objp)')
    #     ax[0].set_xlabel('X Axis')
    #     ax[0].set_ylabel('Y Axis')
    #     ax[0].grid(True)
    #     ax[0].axis('equal')

    #     # 第二个子图：corners 点
    #     ax[1].scatter(imgpoints[-1, :, 0, 0], imgpoints[-1, :, 0, 1], color='red', label='corners (Image Points)')
    #     for i in range(len(imgpoints[0])):
    #         ax[1].text(imgpoints[-1, i, 0, 0], imgpoints[-1, i, 0, 1], f'{i}', color='red', fontsize=12)
    #     ax[1].set_title('Screen Image Points (corners)')
    #     ax[1].set_xlabel('X Axis')
    #     ax[1].set_ylabel('Y Axis')
    #     ax[1].grid(True)
    #     ax[1].axis('equal')

    #     # 调整布局并显示图像
    #     plt.tight_layout()
    #     plt.show()
    
    #print("camera Calibration mtx:", cam_mtx)
    #print("Screen Calibration dist:", cam_dist)
    

    ret, mtx, dist, rvecs, tvecs = cv2.calibrateCamera(objpoints, imgpoints, gray.shape[::-1], cam_mtx, cam_dist)
    # 获得最后一张图片的旋转矩阵和平移向量
    # import pdb; pdb.set_trace()
    R_vector = rvecs[0]
    R = cv2.Rodrigues(R_vector)[0]  # 转换为旋转矩阵
    # R, _ = cv2.Rodrigues(np.array([[0.024044506712974],[0.002032279577832],[-3.138030042895759]]))
    T = tvecs[0]
    # print("*"*100)
    # print("Screen Calibration mtx:", mtx)
    # print("Screen Calibration dist:", dist)
    # print("camera Calibration mtx1:", cam_mtx)
    # print("Screen Calibration dist1:", cam_dist)
   
    # print("*"*100)
  
    mat_data = {
        'screen_matrix': mtx,
        'screen_distortion': dist,
        'screen_rotation_matrix': R,
        'screen_rotation_vector': R_vector,
        'screen_translation_vector': T,
        'screen_error': ret/len(objpoints)
    }
    print('screen mat data = ', mat_data)
    return mat_data



def calibrate_screen_circlegrid(screen_img_path, cam_mtx, cam_dist, chessboard_size, square_size, debug=False):
    # Prepare object points for chessboard corners in world coordinates
    objp = np.zeros((chessboard_size[0] * chessboard_size[1], 3), np.float32)
    objp[:, :2] = np.mgrid[0:chessboard_size[0], 0:chessboard_size[1]].T.reshape(-1, 2)
    objp *= square_size
    # import pdb; pdb.set_trace()

    objpoints = []  # 三维点
    imgpoints = []  # 二维点

    #print('cam_mtx = ', cam_mtx)

    for img_path in screen_img_path:
        
        image_data = np.fromfile(img_path, dtype=np.uint8)
        img = cv2.imdecode(image_data, cv2.IMREAD_COLOR)
       
        # import pdb; pdb.set_trace()
        dst = cv2.undistort(img, cam_mtx, cam_dist, None, None)
        gray = cv2.cvtColor(dst, cv2.COLOR_BGR2GRAY)
        
        circle_w, circle_h = chessboard_size
        criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 30, 0.001)
        params = cv2.SimpleBlobDetector_Params()
        params.maxArea = 10e4
        params.minArea = 10
        params.minDistBetweenBlobs = 5
        blobDetector = cv2.SimpleBlobDetector_create(params)
        ret, corners = cv2.findCirclesGrid(gray, (circle_w, circle_h), cv2.CALIB_CB_SYMMETRIC_GRID, blobDetector, None)
        if ret:
            objpoints.append(objp)
            corners_refined = cv2.cornerSubPix(gray, corners, (circle_w, circle_h), (-1, -1), criteria)
            #cv2.drawChessboardCorners(dst, (circle_w, circle_h), corners_refined, ret)
            imgpoints.append(corners_refined)

            if debug:
                img = cv2.drawChessboardCorners(img, chessboard_size, corners, ret)
                # 在角点图像上标记坐标原点（第一个角点）
                origin = tuple(corners[0].ravel().astype(int))
                cv2.putText(img, '(0,0)', origin, cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)

                # 添加 x 轴箭头 (右方向)
                end_x = tuple(corners[1].ravel().astype(int))  # 选择横向的下一个角点
                cv2.arrowedLine(img, origin, end_x, (255, 255, 255), 2, tipLength=0.2)
                cv2.putText(img, 'X', (end_x[0] + 10, end_x[1]), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)

                # 添加 y 轴箭头 (下方向)
                end_y = tuple(corners[chessboard_size[0]].ravel().astype(int))  # 选择纵向的下一个角点
                cv2.arrowedLine(img, origin, end_y, (255, 255, 255), 2, tipLength=0.2)
                cv2.putText(img, 'Y', (end_y[0], end_y[1] + 30), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255), 2)
                cv2.namedWindow('screen', 0)
                cv2.imshow('screen', img)
                cv2.waitKey(100)
                
    # import pdb; pdb.set_trace()
    cv2.destroyAllWindows()

    objpoints = np.array(objpoints)
    imgpoints = np.array(imgpoints)

    print('objpoints = ', objpoints)
    print('imgpoints = ', imgpoints)
    
   
    ret, mtx, dist, rvecs, tvecs = cv2.calibrateCamera(objpoints, imgpoints, gray.shape[::-1], cam_mtx, cam_dist)
    # 获得最后一张图片的旋转矩阵和平移向量
    # import pdb; pdb.set_trace()
    R_vector = rvecs[-1]
    R = cv2.Rodrigues(R_vector)[0]  # 转换为旋转矩阵
   
    T = tvecs[-1]
  
    mat_data = {
        'screen_matrix': mtx,
        'screen_distortion': dist,
        'screen_rotation_matrix': R,
        'screen_rotation_vector': R_vector,
        'screen_translation_vector': T,
        'screen_error': ret/len(objpoints)
    }
    print('screen mat data = ', mat_data)
    return mat_data



def calibrate_screen_aprilgrid(screen_img_path, cam_mtx, cam_dist, chessboard_size, square_size, square_spacing, debug=False):
    # Prepare object points for chessboard corners in world coordinates
    # objp = np.zeros((chessboard_size[0] * chessboard_size[1], 3), np.float32)
    # objp[:, :2] = np.mgrid[0:chessboard_size[0], 0:chessboard_size[1]].T.reshape(-1, 2)
    # objp *= square_size
    # import pdb; pdb.set_trace()
    # 初始化 AprilTag 检测器
    #detector = Detector('t36h11')
    dll_path = "C:\\Users\\zhang\\AppData\\Local\\Programs\\Python\\Python312\\Lib\\site-packages\\pupil_apriltags\\lib\\apriltag.dll"
    if not os.path.exists(dll_path):
        raise FileNotFoundError(f"{dll_path} 不存在")
    else:
        print('file exists')

    ctypes.WinDLL(dll_path, winmode=0)
    detector = apriltag.Detector(
        families="tag36h11",  # 确保与标签类型一致
        nthreads=4,           # 多线程加速
        quad_decimate=0.5,    # 降采样因子，调小提高检测精度
        quad_sigma=0.8,       # 高斯模糊，处理噪声图像
        refine_edges=True     # 边缘细化
    )
    #detector = Detector('t36h11')

    object_point_single = generate_3d_points(chessboard_size, square_size, square_spacing)  

    #visualize_3d_points(object_point_single)

    # 用于保存所有图像的检测到的角点和 ID
    all_corners = []
    all_object_points = []
    image_size = None
    img_lst = []

    # 遍历每张图像
    for img_file in screen_img_path:
        print('Processing:', img_file)
        
        # 从文件读取并解码图像
        image_data = np.fromfile(img_file, dtype=np.uint8)
        img = cv2.imdecode(image_data, cv2.IMREAD_COLOR)
        img_lst.append(img)
        #screen mirror flip
        #img = cv2.flip(img, 0)

        if image_size is None:
            image_size = (img.shape[1], img.shape[0])  # 保存图像尺寸
        
        # 转换为灰度图像
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        gray = cv2.undistort(gray, cam_mtx, cam_dist, None, cam_mtx)
        
        # 检测 AprilTags
        detections = detector.detect(gray)
        if debug:
            show_detect_result_new(img, detections)
       
        # 如果检测到标签
        if detections:
            image_points = []
            object_points = []

            # 遍历每个检测到的标签
            for detection in detections:
                corners = detection.corners  # 检测到的角点 (4, 2)
                tag_id = detection.tag_id    # 标签 ID

                # 将检测到的角点存储为 np.float32 格式
                image_points.append(corners.astype(np.float32))

                # 根据标签 ID 提取相应的 3D 坐标
                if tag_id < len(object_point_single):
                    object_points.append(object_point_single[tag_id])
                else:
                    print(f"Warning: Tag ID {tag_id} is out of range.")

            if object_points and image_points:
                # 保存当前图像的 2D 角点
                all_corners.append(np.array(image_points, dtype=np.float32).reshape(-1, 2))  # 转为 (N, 2) 形状

                # 保存对应的 3D 坐标
                all_object_points.append(np.array(object_points, dtype=np.float32).reshape(-1, 3))  # 转为 (N, 3) 形状
            else:
                print("No valid detections in this image.")

    # 调用相机标定
    ret, mtx, dist, rvecs, tvecs = cv2.calibrateCamera(
        all_object_points,   # 3D 物体点
        all_corners,         # 2D 图像点
        image_size,          # 图像大小
        None,                # 相机内参初始值
        None                 # 畸变系数初始值
    )

   # 计算重投影误差
    total_error = 0
    for i in range(len(all_object_points)):
        imgpoints2, _ = cv2.projectPoints(all_object_points[i], rvecs[i], tvecs[i], mtx, dist)

        # 将 imgpoints2 从 (N, 1, 2) 转换为 (N, 2)
        imgpoints2 = np.squeeze(imgpoints2)

        # 确保 all_corners[i] 和 imgpoints2 类型匹配
        error = cv2.norm(all_corners[i], imgpoints2, cv2.NORM_L2) / len(imgpoints2)
        total_error += error
        if debug:
            for (x, y), (px, py) in zip(all_corners[i], np.squeeze(imgpoints2)):
                cv2.circle(img_lst[i], (int(x), int(y)), 5, (0, 255, 0), -1)  # Original points in green
                cv2.circle(img_lst[i], (int(px), int(py)), 5, (0, 0, 255), -1)  # Reprojected points in red
            cv2.namedWindow('Error Visualization', 0)
            cv2.imshow('Error Visualization', img_lst[i])
            cv2.waitKey(100)

    mean_error = total_error / len(all_object_points)
    print(f"Mean re-projection error: {mean_error}")

    print('ret/len(all_object_points) = ', ret/len(all_object_points))
    #print('rvecs[0] = ', (rvecs[0]))

    # 获得第一张图片的旋转矩阵和平移向量
    R = cv2.Rodrigues(rvecs[-2])[0]  # 转换为旋转矩阵
    T = tvecs[-2]

    # 创建一个字典来保存所有的标定数据
    calibration_data = {
        'screen_matrix': mtx,
        'screen_distortion': dist,
        'screen_rotation_matrix': R,
        'screen_translation_vector': T,
        'screen_error': ret/len(all_object_points)
    }

   

    # # 创建一个字典来保存所有的标定数据
    # calibration_data = {
    #     'camera_matrix': mtx,
    #     'distortion_coefficients': dist,
    #     'rotation_matrix': rvecs,
    #     'translation_vector': tvecs,
    #     'mean_reprojection_error': mean_error,
    #     'error':ret/len(all_object_points)
    # }
    print(calibration_data)

    return calibration_data
        